function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}
function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor);
  }
}
function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  Object.defineProperty(Constructor, "prototype", {
    writable: false
  });
  return Constructor;
}
function _toPrimitive(input, hint) {
  if (typeof input !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];
  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (typeof res !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (hint === "string" ? String : Number)(input);
}
function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");
  return typeof key === "symbol" ? key : String(key);
}

var generateListFromRange = function generateListFromRange(min, max) {
  var result = [];
  for (var i = min; i <= max; i++) {
    result.push(i);
  }
  return result;
};
var randIntRange = function randIntRange(i, j, engine) {
  var min = Math.ceil(i);
  var max = Math.floor(j);
  return Math.floor(engine() * (max - min + 1)) + min;
};
var sliceOut = function sliceOut(arr, i) {
  return arr.slice(0, i).concat(arr.slice(i + 1));
};
var Aimless = /*#__PURE__*/function () {
  function Aimless() {
    var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : Math.random;
    _classCallCheck(this, Aimless);
    this.engine = engine;
  }

  /**
   * Call the provided engine
   */
  _createClass(Aimless, [{
    key: "call",
    value: function call() {
      return this.engine();
    }

    /**
     * Returns a random number within the integer range
     * @param min Minimum integer
     * @param max Maximum integer
     */
  }, {
    key: "intRange",
    value: function intRange(min, max) {
      return randIntRange(min, max, this.engine);
    }

    /**
     * Returns a random number within the float range
     * @param min Minimum value
     * @param max Maximum value
     */
  }, {
    key: "floatRange",
    value: function floatRange(min, max) {
      return this.engine() * (max - min) + min;
    }

    /**
     * Returns a random number between -1 and 1
     */
  }, {
    key: "normal",
    value: function normal() {
      return this.floatRange(-1, 1);
    }

    /**
     * Returns a random value from the provided array
     * @param arr Array of values
     */
  }, {
    key: "oneOf",
    value: function oneOf(arr) {
      return arr[this.intRange(0, arr.length - 1)];
    }

    /**
     * Returns a new array with randomized order
     * @param arr Array of values
     */
  }, {
    key: "sequence",
    value: function sequence(arr) {
      var result = [];
      var tempArr = arr;
      var i;
      while (result.length < arr.length) {
        i = this.intRange(0, tempArr.length - 1);
        result.push(tempArr[i]);
        tempArr = sliceOut(tempArr, i);
      }
      return result;
    }

    /**
     * Returns an array with every value in the range in random order
     * @param min Minimum value
     * @param max Maximum value
     */
  }, {
    key: "intSequence",
    value: function intSequence(min, max) {
      return this.sequence(generateListFromRange(min, max));
    }

    /**
     * Returns a random boolean (true or false)
     */
  }, {
    key: "bool",
    value: function bool() {
      return !!this.intRange(0, 1);
    }

    /**
     * Returns a random sign (-1 or 1)
     */
  }, {
    key: "sign",
    value: function sign() {
      return this.bool() ? 1 : -1;
    }

    /**
     * Returns a random character from the provided array
     * @param str String
     */
  }, {
    key: "char",
    value: function char(str) {
      return str[this.intRange(0, str.length - 1)];
    }

    /**
     * Returns a random value from the provided array, biased towards provided weights
     * @param nums Array of numbers (integer or float)
     * @param weights Array of weights (integer or float)
     */
  }, {
    key: "weighted",
    value: function weighted(nums, weights) {
      if (nums.length !== weights.length) {
        throw new Error('Every provided number must have a corresponding weight');
      }
      var totalWeight = weights.reduce(function (sum, weight) {
        return sum + weight;
      }, 0);
      var rand = this.engine() * totalWeight;
      var cumulativeWeight = 0;
      var selectedIndex = 0;
      for (var i = 0; i < nums.length; i++) {
        cumulativeWeight += weights[i];
        if (rand < cumulativeWeight) {
          selectedIndex = i;
          break;
        }
      }
      return nums[selectedIndex];
    }

    /**
     * Returns a random value that follows a normal distribution
     * @param mean Mean
     * @param stdDev Standard Deviation
     */
  }, {
    key: "normalDist",
    value: function normalDist(mean, stdDev) {
      // Box-Muller transform
      var u, v, s;
      do {
        u = this.engine() * 2 - 1;
        v = this.engine() * 2 - 1;
        s = Math.pow(u, 2) + Math.pow(v, 2);
      } while (s === 0 || s >= 1);
      var t = Math.sqrt(-2 * Math.log(s) / s);
      var rand = mean + stdDev * u * t;
      return rand;
    }

    /**
     * Returns a random value that follows an exponential distribution
     * @param lambda Lambda
     */
  }, {
    key: "exponentialDist",
    value: function exponentialDist(lambda) {
      return -Math.log(1 - this.engine()) / lambda;
    }

    /**
     * Returns a random value that follows a custom distribution
     * @param func Distribution function that accepts a random number between 0 and 1
     */
  }, {
    key: "customDist",
    value: function customDist(func) {
      return func(this.engine());
    }

    /**
     * Returns a valid RFC4122 version4 ID hex string, using the provided engine
     */
  }, {
    key: "uuid",
    value: function uuid() {
      // Credit @Alexey Silin from https://gist.github.com/1308368
      var a = '';
      var b = '';
      for (b = a = ''; a++ < 36; b += ~a % 5 | a * 3 & 4 ? (a ^ 15 ? 8 ^ this.engine() * (a ^ 20 ? 16 : 4) : 4).toString(16) : '-') {}
      return b;
    }

    /**
     * Returns a seeded PRNG function
     * @param seed Seed (integer)
     */
  }], [{
    key: "seedFunc",
    value: function seedFunc(seed) {
      // Park-Miller PRNG
      var currentSeed = seed % 2147483647;
      return function () {
        currentSeed = currentSeed * 16807 % 2147483647;
        return (currentSeed - 1) / 2147483646;
      };
    }

    /**
     * Returns a PRNG function that returns unique values from the provided array.
     * @param arr Array of values to pull from
     * @param engine Engine to be used as PRNG
     */
  }, {
    key: "uniqFuncSequence",
    value: function uniqFuncSequence(arr) {
      var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Math.random;
      var tempArr = arr;
      return function () {
        if (!tempArr.length) return null;
        var i = randIntRange(0, tempArr.length - 1, engine);
        var result = tempArr[i];
        tempArr = sliceOut(tempArr, i);
        return result;
      };
    }

    /**
     * Returns a PRNG function that returns unique values from the provided range.
     * @param min Minimum integer
     * @param max Maximum integer
     * @param engine Engine to be used as PRNG
     */
  }, {
    key: "uniqFuncIntRange",
    value: function uniqFuncIntRange(min, max) {
      var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : Math.random;
      return this.uniqFuncSequence(generateListFromRange(min, max), engine);
    }
  }]);
  return Aimless;
}();

export default Aimless;
